Package: plotgardener
Title: Coordinate-Based Genomic Visualization Package for R
Version: 1.17.0
Date: 2024-06-28
Author: Nicole Kramer [aut, cre], Eric S. Davis [aut], Craig Wenger [aut],
    Sarah Parker [ctb], Erika Deoudes [art], Michael Love [ctb],
    Douglas H. Phanstiel [aut, cre, cph]
Maintainer: Nicole Kramer <nekramer27@gmail.com>,
 Douglas Phanstiel <douglas_phanstiel@med.unc.edu>
Description: 
    Coordinate-based genomic visualization package for R. It grants users
    the ability to programmatically produce complex, multi-paneled figures. Tailored for
    genomics, plotgardener allows users to visualize large complex genomic datasets and
    provides exquisite control over how plots are placed and arranged on a page. 
Depends: R (>= 4.2.0)
License: MIT + file LICENSE
URL: https://phanstiellab.github.io/plotgardener,
        https://github.com/PhanstielLab/plotgardener
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: curl, data.table, dplyr, GenomeInfoDb, GenomicRanges, glue,
        grDevices, grid, ggplotify, IRanges, methods, plyranges, purrr,
        Rcpp, RColorBrewer, rhdf5, rlang, stats, strawr, tools, utils,
        withr
biocViews: Visualization, GenomeAnnotation, FunctionalGenomics,
        GenomeAssembly, HiC
LinkingTo: Rcpp
Suggests: AnnotationDbi, AnnotationHub, BSgenome,
        BSgenome.Hsapiens.UCSC.hg19, ComplexHeatmap, GenomicFeatures,
        ggplot2, InteractionSet, knitr, org.Hs.eg.db, rtracklayer,
        plotgardenerData, pdftools, png, rmarkdown, scales, showtext,
        testthat (>= 3.0.0), TxDb.Hsapiens.UCSC.hg19.knownGene,
        TxDb.Hsapiens.UCSC.hg38.knownGene
VignetteBuilder: knitr
Config/testthat/edition: 3
BugReports: https://github.com/PhanstielLab/plotgardener/issues
git_url: https://git.bioconductor.org/packages/plotgardener
git_branch: devel
git_last_commit: eaa9c96
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
NeedsCompilation: yes
Packaged: 2025-11-05 00:14:31 UTC; biocbuild
