## Download font files found in inst/extdata

## Regular
download.file(
    url = "https://www.download-free-fonts.com/details/96003/proxima-nova",
    destfile = "inst/extdata/proximanova-regular.otf")

## Semi-bold
download.file(
    url = "https://www.download-free-fonts.com/details/92405/proxima-nova-semibold",
    destfile = "inst/extdata/proximanova-semibold.otf")

## Regular italic
download.file(
    url = "https://www.download-free-fonts.com/details/93063/proxima-nova-regular-italic",
    destfile = "inst/extdata/proximanova-regitalic.otf")
