# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

checkRow <- function(x, maxRows, rowCol, wig) {
    .Call('_plotgardener_checkRow', PACKAGE = 'plotgardener', x, maxRows, rowCol, wig)
}

rebinBigwig <- function(bigwig, bins) {
    .Call('_plotgardener_rebinBigwig', PACKAGE = 'plotgardener', bigwig, bins)
}

